package digital.laboratory.platform.imr.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.imr.component.DateUtils;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * 毒品库中的毒品检材信息 VO 返回前台显示类
 * @TableName b_drug_material_info
 */
@Data
@ApiModel(value = "DrugMaterialInfoVO", description = "毒品库中的毒品检材信息 VO 返回前台显示类")
public class DrugMaterialInfoVO {
    /**
     * 主键标识
     */
    @ApiModelProperty("主键标识")
    private String id;

    /**
     * 关联的案件id 
     */
    @ApiModelProperty("关联的案件id")
    private String caseId;

    /**
     * 关联的入库记录id
     */
    @ApiModelProperty("关联的入库记录id")
    private String sampleStorageId;

    @ApiModelProperty("存放位置")
    private String storageLocation;

    /**
     * 毒品检材编号
     */
    @ApiModelProperty("毒品检材编号")
    private String drugNo;

    /**
     * 毒品检材名称
     */
    @ApiModelProperty("毒品检材名称")
    private String drugName;

    /**
     * 质量/体积
     */
    @ApiModelProperty("质量/体积")
    private String massVolume;

    /**
     * 质量/体积 单位
     */
    @ApiModelProperty("质量/体积 单位")
    private String unit;

    /**
     * 包装是否完整， 1 完整 | 0 不完整
     */
    @ApiModelProperty("包装是否完整， 1 完整 | 0 不完整")
    private Boolean packageComplete;

    /**
     * 毒品检材状态， 0 录入信息中 | 3 待入库 | 5 已入库
     */
    @ApiModelProperty("毒品检材状态， 0 录入信息中 | 3 待入库 | 5 已入库")
    private Integer status;

    /**
     * 毒品检材状态， 0 录入信息中 | 3 待入库 | 5 已入库
     */
    @ApiModelProperty("毒品检材状态， 0 录入信息中 | 3 待入库 | 5 已入库")
    private String statusName;

    /**
     * 案事件名称
     */
    @ApiModelProperty("案事件名称")
    private String caseName;

    /**
     * 案件编号
     */
    @ApiModelProperty("案件编号")
    private String caseNo;

    /**
     * 送缴单位
     */
    @ApiModelProperty("送缴单位")
    private String handingOverOrg;

    /**
     * 送缴单位
     */
    @ApiModelProperty("送缴单位")
    private String handingOverOrgName;

    /**
     * 送缴日期
     */
    @ApiModelProperty("送缴日期")
    @JsonFormat(pattern = DateUtils.yyyy_MM_dd)
    private LocalDate handingOverDate;

}